@@ -132,16 +132,30 @@ public:
 
   void BeginUpdate() { ++mUpdateCount; }
   void EndUpdate();
+  void RecalcQuotesAndCounters();
 
   void WillDestroyFrameTree();
 
   // Note: It's the caller's responsibility to make sure to wrap a
   // ProcessRestyledFrames call in a view update batch.
+  // This function does not call ProcessAttachedQueue() on the binding manager.
+  // If the caller wants that to happen synchronously, it needs to handle that
+  // itself.
   nsresult ProcessRestyledFrames(nsStyleChangeList& aRestyleArray);
 
+private:
+  // This function does not call ProcessAttachedQueue() on the binding manager.
+  // If the caller wants that to happen synchronously, it needs to handle that
+  // itself.
   void ProcessOneRestyle(nsIContent* aContent, nsReStyleHint aRestyleHint,
                          nsChangeHint aChangeHint);
+
+public:
+  // This function does not call ProcessAttachedQueue() on the binding manager.
+  // If the caller wants that to happen synchronously, it needs to handle that
+  // itself.
   void ProcessPendingRestyles();
+
   void PostRestyleEvent(nsIContent* aContent, nsReStyleHint aRestyleHint,
                         nsChangeHint aMinChangeHint);
 
@@ -250,7 +264,8 @@ private:
   nsresult AppendFrames(const nsFrameConstructorState& aState,
                         nsIContent*                    aContainer,
                         nsIFrame*                      aParentFrame,
-                        nsIFrame*                      aFrameList);
+                        nsIFrame*                      aFrameList,
+                        nsIFrame*                      aAfterFrame);
 
   // BEGIN TABLE SECTION
   /**
@@ -259,7 +274,9 @@ private:
    * the child list of the outer frame, and will put any pseudo frames it had
    * to create into aChildItems.  The newly-created outer frame will either be
    * in aChildItems or a descendant of a pseudo in aChildItems (unless it's
-   * positioned, in which case its placeholder will be in aChildItems).
+   * positioned or floated, in which case its placeholder will be in
+   * aChildItems).  If aAllowOutOfFlow is false, the table frame will be forced
+   * to be in-flow no matter what its float or position values are.
    */ 
   nsresult ConstructTableFrame(nsFrameConstructorState& aState,
                                nsIContent*              aContent,
@@ -268,6 +285,7 @@ private:
                                nsTableCreator&          aTableCreator,
                                PRBool                   aIsPseudo,
                                nsFrameItems&            aChildItems,
+                               PRBool                   aAllowOutOfFlow,
                                nsIFrame*&               aNewOuterFrame,
                                nsIFrame*&               aNewInnerFrame);
 
@@ -418,6 +436,7 @@ private:
                              const nsStyleDisplay* aChildDisplay,
                              nsIAtom* aTag,
                              PRInt32 aNameSpaceID,
+                             nsStyleContext* aChildStyle,
                              nsIFrame* & aParentFrame,
                              nsFrameItems* & aFrameItems,
                              nsFrameConstructorState& aState,
@@ -735,6 +754,10 @@ private:
   PRBool HaveFirstLetterStyle(nsIContent*      aContent,
                               nsStyleContext*  aStyleContext);
 
+  // Check whether a given block has first-letter style.  Make sure to
+  // only pass in blocks!  And don't pass in null either.
+  PRBool HaveFirstLetterStyle(nsIFrame* aBlockFrame);
+
   PRBool HaveFirstLineStyle(nsIContent*      aContent,
                             nsStyleContext*  aStyleContext);
 
@@ -1007,8 +1030,10 @@ private:
   nsQuoteList         mQuoteList;
   nsCounterManager    mCounterManager;
   PRUint16            mUpdateCount;
-  PRPackedBool        mQuotesDirty;
-  PRPackedBool        mCountersDirty;
+  PRPackedBool        mQuotesDirty : 1;
+  PRPackedBool        mCountersDirty : 1;
+  PRPackedBool        mInitialContainingBlockIsAbsPosContainer : 1;
+  PRPackedBool        mIsDestroyingFrameTree : 1;
 
   nsCOMPtr<nsILayoutHistoryState> mTempFrameTreeState;
 
