@@ -1039,6 +1039,8 @@ function loadStartFolder(initialUri)
       // this is the case where we're trying to auto-subscribe to a folder.
       if (initialUri && !startFolder.parent)
       {
+        // hack to force display of thread pane.
+        ShowingThreadPane();
         messenger.loadURL(window, initialUri);
         return;
       }
@@ -1739,11 +1741,6 @@ function SelectMessage(messageUri)
     gDBView.selectMsgByKey(msgHdr.messageKey);
 }
 
-function ReloadWithAllParts()
-{
-  gDBView.reloadMessageWithAllParts();
-}
-
 function ReloadMessage()
 {
   gDBView.reloadMessage();
